@@ -10,7 +10,7 @@ import posixpath
 import re
 import warnings
 import weakref
-from datetime import datetime, timedelta, timezone
+from datetime import datetime
 from urllib.parse import parse_qs
 from urllib.parse import quote as quote_urllib
 from urllib.parse import urlsplit
@@ -149,7 +149,7 @@ class GCSFileSystem(AsyncFileSystem):
       metadata service, anonymous.
     - ``token='google_default'``, your default gcloud credentials will be used,
       which are typically established by doing ``gcloud login`` in a terminal.
-    - ``token=='cache'``, credentials from previously successful gcsfs
+    - ``token='cache'``, credentials from previously successful gcsfs
       authentication will be used (use this after "browser" auth succeeded)
     - ``token='anon'``, no authentication is performed, and you can only
       access data which is accessible to allUsers (in this case, the project and
@@ -166,10 +166,10 @@ class GCSFileSystem(AsyncFileSystem):
       or a Credentials object. gcloud typically stores its tokens in locations
       such as
       ``~/.config/gcloud/application_default_credentials.json``,
-      `` ~/.config/gcloud/credentials``, or
+      ``~/.config/gcloud/credentials``, or
       ``~\AppData\Roaming\gcloud\credentials``, etc.
 
-    Specific methods, (eg. `ls`, `info`, ...) may return object details from GCS.
+    Specific methods, (eg. ``ls``, ``info``, ...) may return object details from GCS.
     These detailed listings include the
     [object resource](https://cloud.google.com/storage/docs/json_api/v1/objects#resource)
 
@@ -199,8 +199,8 @@ class GCSFileSystem(AsyncFileSystem):
     created via other processes *will not* be visible to the GCSFileSystem until the cache
     refreshed. Calls to GCSFileSystem.open and calls to GCSFile are not effected by this cache.
 
-    In the default case the cache is never expired. This may be controlled via the `cache_timeout`
-    GCSFileSystem parameter or via explicit calls to `GCSFileSystem.invalidate_cache`.
+    In the default case the cache is never expired. This may be controlled via the ``cache_timeout``
+    GCSFileSystem parameter or via explicit calls to ``GCSFileSystem.invalidate_cache``.
 
     Parameters
     ----------
@@ -225,11 +225,11 @@ class GCSFileSystem(AsyncFileSystem):
     secure_serialize: bool (deprecated)
     requester_pays : bool, or str default False
         Whether to use requester-pays requests. This will include your
-        project ID `project` in requests as the `userPorject`, and you'll be
+        project ID `project` in requests as the `userProject`, and you'll be
         billed for accessing data from requester-pays buckets. Optionally,
         pass a project-id here as a string to use that as the `userProject`.
     session_kwargs: dict
-        passed on to aiohttp.ClientSession; can contain, for example,
+        passed on to ``aiohttp.ClientSession``; can contain, for example,
         proxy settings.
     endpoint_url: str
         If given, use this URL (format protocol://host:port , *without* any
@@ -745,10 +745,9 @@ class GCSFileSystem(AsyncFileSystem):
 
     def _parse_timestamp(self, timestamp):
         assert timestamp.endswith("Z")
-        UTC = timezone(timedelta(0))
         timestamp = timestamp[:-1]
         timestamp = timestamp + "0" * (6 - len(timestamp.rsplit(".", 1)[1]))
-        return datetime.fromisoformat(timestamp).astimezone(UTC)
+        return datetime.fromisoformat(timestamp + "+00:00")
 
     async def _info(self, path, generation=None, **kwargs):
         """File information about this path."""
@@ -835,15 +834,10 @@ class GCSFileSystem(AsyncFileSystem):
             for entry in await self._list_objects(
                 path, prefix=prefix, versions=versions
             ):
-                if versions:
-                    out.append(
-                        {
-                            **entry,
-                            "name": f"{entry['name']}#{entry['generation']}",
-                        }
-                    )
-                else:
-                    out.append(entry)
+                if versions and "generation" in entry:
+                    entry = entry.copy()
+                    entry["name"] = f"{entry['name']}#{entry['generation']}"
+                out.append(entry)
 
         if detail:
             return out
@@ -893,7 +887,7 @@ class GCSFileSystem(AsyncFileSystem):
         fake-gcs-server:latest does not seem to support this.
 
         Parameters
-        ---------
+        ----------
         content_type: str
             If not None, set the content-type to this value
         content_encoding: str
@@ -907,6 +901,7 @@ class GCSFileSystem(AsyncFileSystem):
                 - content_encoding
                 - content_language
                 - custom_time
+
             More info:
             https://cloud.google.com/storage/docs/metadata#mutable
         kw_args: key-value pairs like field="value" or field=None
@@ -1225,7 +1220,9 @@ class GCSFileSystem(AsyncFileSystem):
                     "size": 0,
                 }
 
-                cache_entries.setdefault(parent, []).append(previous)
+                listing = cache_entries.setdefault(parent, [])
+                if previous not in listing:
+                    listing.append(previous)
 
                 previous = dirs[parent]
                 parent = self._parent(parent)
@@ -1685,7 +1682,7 @@ async def upload_chunk(fs, location, data, offset, size, content_type):
     range = "bytes %i-%i/%i" % (offset, offset + l - 1, size)
     head["Content-Range"] = range
     head.update({"Content-Type": content_type, "Content-Length": str(l)})
-    headers, txt = await fs._call("POST", location, headers=head, data=data)
+    headers, txt = await fs._call("POST", location, headers=head, data=io.BytesIO(data))
     if "Range" in headers:
         end = int(headers["Range"].split("-")[1])
         shortfall = (offset + l - 1) - end
@@ -1751,7 +1748,7 @@ async def simple_upload(
         path,
         uploadType="multipart",
         headers={"Content-Type": 'multipart/related; boundary="==0=="'},
-        data=data,
+        data=io.BytesIO(data),
         json_out=True,
     )
     checker.update(datain)
